# Authentication

The CoinsSend API uses a signature-based authentication system rather than traditional token or session-based authentication. All API requests to endpoints requiring merchant authentication must include two headers:

- `Merchant`: Your merchant ID
- `Sign`: Signature for request verification

This signature-based system ensures that your API requests are secure, verifiable, and cannot be tampered with during transmission.

## API Keys

Before you can use the API, you need to obtain an API key from your CoinsSend merchant dashboard. API keys have specific permissions:

- `is_allowed_withdraw` - Permission to initiate withdrawals

Ensure that your API key has the necessary permissions for the operations you want to perform.

## Signature Generation

The `Sign` header is a signature that verifies your request is authentic and has not been tampered with. To generate this signature:

1. Convert your request body to a JSON string
2. Encode the JSON string using Base64
3. Concatenate the Base64-encoded string with your API key
4. Calculate the MD5 hash of the concatenated string

## Code Examples

<div datatype="code-tabs">

```php
<?php
function generateSignature($data, $apiKey) {
    // Convert request data to JSON
    $jsonData = json_encode($data);
    
    // Base64 encode the JSON string
    $base64Data = base64_encode($jsonData);
    
    // Concatenate with API key and calculate MD5 hash
    $signature = md5($base64Data . $apiKey);
    
    return $signature;
}

// Usage
$requestData = [
    'order_id' => 'order_12345',
    'amount' => '100.50'
];

$apiKey = 'your_api_key_here';
$signature = generateSignature($requestData, $apiKey);

// Now use this in your request headers
// Merchant: your_merchant_id
// Sign: $signature
```

```javascript
const crypto = require('crypto');

function generateSignature(data, apiKey) {
    // Convert request data to JSON
    const jsonData = JSON.stringify(data);
    
    // Base64 encode the JSON string
    const base64Data = Buffer.from(jsonData).toString('base64');
    
    // Concatenate with API key and calculate MD5 hash
    const signature = crypto.createHash('md5').update(base64Data + apiKey).digest('hex');
    
    return signature;
}

// Usage
const requestData = {
    order_id: 'order_12345',
    amount: 100.50
};

const apiKey = 'your_api_key_here';
const signature = generateSignature(requestData, apiKey);

// Now use this in your request headers
// Merchant: your_merchant_id
// Sign: signature
```

```python
import json
import base64
import hashlib

def generate_signature(data, api_key):
    # Convert request data to JSON
    json_data = json.dumps(data)
    
    # Base64 encode the JSON string
    base64_data = base64.b64encode(json_data.encode()).decode()
    
    # Concatenate with API key and calculate MD5 hash
    signature = hashlib.md5((base64_data + api_key).encode()).hexdigest()
    
    return signature

# Usage
request_data = {
    'order_id': 'order_12345',
    'amount': '100.50'
}

api_key = 'your_api_key_here'
signature = generate_signature(request_data, api_key)

# Now use this in your request headers
# Merchant: your_merchant_id
# Sign: signature
```


</div>

### GET Request Example (JavaScript)

For `GET` requests, pass `null` as the request body when generating the signature.

```javascript
const crypto = require('crypto')

function generateSignature(data, apiKey) {
  const jsonData = data === null ? [] : JSON.stringify(data)
  const base64Data = Buffer.from(jsonData).toString('base64')
  return crypto.createHash('md5').update(base64Data + apiKey).digest('hex')
}

const merchantId = 'your_merchant_id'
const apiKey = 'your_api_key_here'
const invoiceCode = 'invoice_12345'

const signature = generateSignature(null, apiKey)

fetch(`https://api.coinssend.com/v1/invoices/${invoiceCode}`, {
  method: 'GET',
  headers: {
    'Content-Type': 'application/json',
    'Merchant': merchantId,
    'Sign': signature
  }
})
  .then(res => res.json())
  .then(data => console.log(data))
  .catch(err => console.error('Request failed:', err))
```

```typescript
function generateSignature(data: unknown, apiKey: string): string {
  const jsonData = data === null ? [] : JSON.stringify(data)
  const base64Data = Buffer.from(jsonData).toString('base64')
  const signature = crypto
    .createHash('md5')
    .update(base64Data + apiKey)
    .digest('hex')
  return signature
}
```

## Common Authentication Errors

| HTTP Code | Error Code | Description |
|-----------|------------|-------------|
| 400 | MISSING_HEADERS | Either Merchant or Sign header is missing |
| 401 | INVALID_MERCHANT | The provided merchant ID doesn't exist |
| 401 | INVALID_SIGN | The signature verification failed |

## Best Practices

1. **Keep Your API Key Secure**: Never expose your API key in client-side code or public repositories.

2. **Regenerate Keys If Compromised**: If you suspect your API key has been compromised, regenerate it immediately from your merchant dashboard.

3. **Implement Key Rotation**: Regularly rotate your API keys for improved security.

4. **Use HTTPS**: Always use HTTPS to ensure that your requests and API keys are encrypted during transmission.

5. **Check Response Status**: Always validate the response status codes and messages to ensure your authentication is working correctly.